from django.urls import path
from . import views

urlpatterns = [
    path(r'user/', views.UserList.as_view({"get": "list"}), name="Get User List"),
    path(r'user/create/', views.UserCreate.as_view({"post": "create"}), name="Create One User"),
    path(r'user/setpermission/', views.UserPermission.as_view({"post": "create"}), name="Set Permission For User"),
    path(r'user/changepwd/', views.UserChangePWD.as_view({"post": "create"}), name="Change Password"),
    path(r'user/lock/', views.UserLock.as_view({"post": "create"}), name="Lock & Unlock User"),
    path(r'user/delete/', views.UserDelete.as_view({"post": "create"}), name="Delete One User"),
]



from os.path import join, exists
from os import makedirs, getcwd
import shutil
from pathlib import Path
import toml
from configparser import ConfigParser


def create_file(folder: str):
    """
    create file
    :return:
    """
    current_path = Path(__file__).resolve()
    working_space = join(getcwd())
    file_path = join(current_path.parent, 'file')
    if folder == '':
        folder = 'bomiot'
    if exists(join(working_space, 'setup.ini')) is False:
        shutil.copy2(join(file_path, 'setup.ini'), working_space)
        if folder != 'bomiot':
            setup_config = ConfigParser()
            setup_config.read(join(working_space, 'setup.ini'), encoding='utf-8')
            setup_config.set('project', 'name', folder)
            setup_config.write(open(join(working_space, 'setup.ini'), "wt"))
    if exists(join(working_space, 'pyproject.toml')) is False:
        if folder != 'bmoiot':
            with open(join(file_path, 'pyproject.toml'), 'r', encoding='utf-8') as pip_file:
                deploy_pip = toml.load(pip_file)
            deploy_pip['tool']['poetry']['name'] = folder
            deploy_pip['tool']['poetry']['version'] = '0.0.1'
            with open(join(working_space, 'pyproject.toml'), 'w', encoding='utf-8') as user_pip_file:
                toml.dump(deploy_pip, user_pip_file)

    if exists(join(working_space, '.gitignore')) is False:
        shutil.copy2(join(file_path, '.gitignore'), working_space)

    if exists(join(working_space, 'LICENSE')) is False:
        shutil.copy2(join(file_path, 'LICENSE'), working_space)

    if exists(join(working_space, 'README.md')) is False:
        shutil.copy2(join(file_path, 'README.md'), working_space)

    log_path = join(working_space, 'logs')
    exists(log_path) or makedirs(log_path)

    deploy_path = join(working_space, 'deploy')
    exists(deploy_path) or makedirs(deploy_path)

    working_config = ConfigParser()
    working_path = join(join(current_path.parent.parent, 'server'), 'workspace.ini')
    working_config.read(working_path, encoding='utf-8')
    working_config.set('space', 'name', working_space)
    working_config.write(open(working_path, "wt"))

    print("Welcome to bomiot")


from os.path import join, exists
from os import makedirs, getcwd
import os
import sys
import shutil
from pathlib import Path
from .init import create_file
from configparser import ConfigParser
import pkg_resources
from .copyfile import copy_files
from .changeapps import create_plugins_apps_py


def plugins(folder: str):
    """
    plugins workspace
    :param folder:
    :return:
    """
    if len(sys.argv) < 3:
        print('Please enter your plugins name')
    else:
        plugins_path = join(getcwd(), sys.argv[2])
        if exists(plugins_path):
            print('Plugins directory already exists')
        else:
            if sys.argv[2] in [pkg.key for pkg in pkg_resources.working_set]:
                print('Plugins directory already exists')
            else:
                os.makedirs(plugins_path)
                current_path = Path(__file__).resolve()
                file_path = join(current_path.parent, 'file')

                shutil.copy2(join(file_path, '__version__.py'), plugins_path)

                with open(join(plugins_path, '__init__.py'), "w") as f:
                    f.write("def version():\n")
                    f.write(f"    from {sys.argv[2]} import __version__\n")
                    f.write("    return __version__.version()\n")
                f.close()

                shutil.copy2(join(file_path, 'bomiotconf.ini'), plugins_path)

                plugins_config = ConfigParser()
                plugins_config.read(join(plugins_path, 'bomiotconf.ini'), encoding='utf-8')
                plugins_config.set('mode', 'name', 'plugins')
                plugins_config.write(open(join(plugins_path, 'bomiotconf.ini'), 'wt'))

                create_file('')

                copy_files(join(current_path.parent, 'extends'), plugins_path)

                apps_path = join(plugins_path, 'apps.py')
                os.remove(apps_path)
                create_plugins_apps_py(apps_path, sys.argv[2])

                print(f'Initialized plugins workspace {sys.argv[2]}')



import toml
from os.path import join
from django.conf import settings


def permission_message_return(language: str, data: str) -> str:
    with open(join(settings.LANGUAGE_DIR, language + '.toml'), 'r', encoding='utf-8') as message:
        message_data = toml.load(message)
    return message_data['permission'][data]

def detail_message_return(language: str, data: str) -> str:
    print(settings.LANGUAGE_DIR)
    return 'True'

def msg_message_return(language: str, data: str) -> dict:
    print(settings.LANGUAGE_DIR)
    return {'msg': 'True'}



import datetime, hashlib, random
from django.db import models

DEFAULT_DATA = (
    "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ_abcdefghijklmnopqrstuvwxyz-"  # noqa: 501
)


def md5_id():
    ctime = str(datetime.datetime.now())
    m = hashlib.md5(bytes(''.join(random.sample(DEFAULT_DATA, 21)), encoding="utf-8"))
    m.update(bytes(ctime, encoding="utf-8"))
    return m.hexdigest()


class Md5idField(models.CharField):
    def __init__(self, *args, **kwargs) -> None:
        kwargs["max_length"] = kwargs.pop("max_length", 255)
        kwargs["default"] = md5_id()
        super(Md5idField, self).__init__(*args, **kwargs)

    def get_internal_type(self):
        return "CharField"

    def deconstruct(self):
        name, path, args, kwargs = super().deconstruct()
        kwargs['max_length'] = self.max_length
        kwargs.pop('default', None)
        return name, path, args, kwargs


import inspect
import time
from threading import Thread
import json
import importlib
from django.dispatch import receiver
from bomiot.server.core.signal import bomiot_signals
from django_apscheduler.models import DjangoJob
from apscheduler.schedulers.background import BackgroundScheduler
from django_apscheduler.jobstores import DjangoJobStore, register_events
from bomiot.server.core.models import JobList

job_defaults = {
  'max_instances': 10,
  'misfire_grace_time': 9
}
scheduler = BackgroundScheduler(job_defaults=job_defaults)
scheduler.add_jobstore(DjangoJobStore(), 'default', engine_options={'pool_pre_ping': True, 'pool_recycle': 25200})

# map the args
args_map = {
    'cron': ['year', 'month', 'day', 'week', 'day_of_week', 'hour', 'minute', 'second', 'start_date', 'end_date',
             'timezone'],
    'interval': ['weeks', 'days', 'hours', 'minutes', 'seconds', 'start_date', 'end_date', 'timezone'],
    'date': ['run_date', 'timezone']
}


@receiver(bomiot_signals)
def sm_send_success_signal_handler(sender, **kwargs):
    model_data = kwargs['msg'].get('models', '')
    if model_data == 'JobList':
        job_id = f"{inspect.getmodule(sender).__name__}-{sender.__name__}"
        data = kwargs['msg'].get('data', '')
        if JobList.objects.filter(job_id=job_id).exists():
            job_detail = JobList.objects.get(job_id=job_id)
            job_detail.trigger = data.get('trigger')
            job_detail.description = data.get('description', '')
            job_detail.configuration = json.dumps(data)
            job_detail.save()
        else:
            JobList.objects.create(job_id=job_id,
                                   module_name=inspect.getmodule(sender).__name__,
                                   func_name=sender.__name__,
                                   trigger=data.get('trigger'),
                                   description=data.get('description', ''),
                                   configuration=json.dumps(data)
                                   )


class SchedulerManager(Thread):
    """
    Scheduler of Job
    """

    def __init__(self, scheduler):
        """
        init manager
        :param scheduler:
        """
        super(SchedulerManager, self).__init__()
        self.scheduler = scheduler
        register_events(self.scheduler)
        self.setDaemon(True)
        self.scheduler.start()

    def existed_jobs(self):
        """
        get existed jobs stored in db
        :return:
        """
        jobs = DjangoJob.objects.all()
        return jobs

    def realtime_jobs(self):
        """
        get real-time jobs from db
        :return:
        """
        jobs = JobList.objects.filter(type=True)
        return jobs

    def realtime_job_id(self):
        """
        get real-time jobs
        :return:
        """
        jobs = self.realtime_jobs()
        for job in jobs:
            yield job.job_id

    def sync_jobs(self, force=False):
        """
        sync jobs
        :return:
        """
        jobs = self.realtime_jobs()
        for job in jobs:
            # add new jobs or modify existed jobs
            self._add_or_modify_new_jobs(job, force)
            # remove deleted jobs
            self._remove_deprecated_jobs(job, force)


    def _remove_deprecated_jobs(self, job, force=False):
        """
        remove jobs
        :return:
        """
        if not job.type and not force:
            return
        # check extra jobs which does not belong to task
        try:
            existed_jobs = self.existed_jobs()
            existed_job_ids = list(map(lambda obj: obj.id, existed_jobs))
            realtime_job_ids = list(self.realtime_job_id())
            deprecated_job_ids = [
                job_id for job_id in existed_job_ids if not job_id in realtime_job_ids]
            if deprecated_job_ids:
                # remove deprecated jobs
                for job_id in deprecated_job_ids:
                    self.scheduler.remove_job(job_id)
        except:
            pass

    def _add_or_modify_new_jobs(self, job, force=False):
        """
        add new jobs or modify existed jobs
        :return:
        """
        if not job.type and not force:
            return

        # get job id
        job_id = job.job_id
        # add job_id to array
        configuration = json.loads(job.configuration)
        trigger = job.trigger
        configuration = {arg: configuration.get(arg) for arg in args_map.get(trigger) if
                         configuration.get(arg)}
        try:
            job_func = importlib.import_module(f'{job.module_name}')
            job_function = getattr(job_func, job.func_name)
            # if job doesn't exist, add it. otherwise replace it
            self.scheduler.add_job(job_function, job.trigger, id=job_id, replace_existing=True, **configuration)
        except:
            JobList.objects.filter(job_id=job.job_id).delete()



    def run(self):
        """
        heart beat detect
        :return:
        """
        self.sync_jobs(force=True)
        while True:
            self.sync_jobs()
            time.sleep(3)


# init scheduler manager
sm = SchedulerManager(scheduler)



from os.path import join, exists
from os import makedirs, getcwd, rename
import shutil
from pathlib import Path
from .init import create_file
from configparser import ConfigParser
import sys
import os


def deploy(folder: str):
    """
    deploy project
    :param folder:
    :return:
    """

    if len(sys.argv) < 3:
        print('Please enter your deploy project name')
    else:
        current_path = Path(__file__).resolve()
        file_path = join(current_path.parent, 'file')
        create_file('')
        exists(join(getcwd(), 'deploy')) or makedirs(join(getcwd(), 'deploy'))
        deploy_path = join(getcwd(), 'deploy')
        supervisor_path = join(deploy_path, 'supervisor')
        exists(supervisor_path) or os.makedirs(supervisor_path)

        if exists(join(supervisor_path, str(sys.argv[2]) + '.ini')) is False:
            shutil.copy2(join(file_path, 'supervisor.conf'), supervisor_path)
            if exists(join(supervisor_path, str(sys.argv[2]) + '.conf')):
                os.remove(join(supervisor_path, str(sys.argv[2]) + '.conf'))
            rename(join(supervisor_path, 'supervisor.conf'), join(supervisor_path, str(sys.argv[2]) + '.conf'))
        supervisor_config = ConfigParser()
        supervisor_config.read(join(supervisor_path, str(sys.argv[2]) + '.conf'))
        if str(sys.argv[2]) != 'bomiot':
            supervisor_config.add_section(f'program:{sys.argv[2]}')
            supervisor_config.set(f'program:{sys.argv[2]}', 'user', 'root')
            supervisor_config.set(f'program:{sys.argv[2]}', 'command', 'daphne -b 0.0.0.0 -p 8008 bomiot.server.server.asgi:application')
            supervisor_config.set(f'program:{sys.argv[2]}', 'autostart', 'true')
            supervisor_config.set(f'program:{sys.argv[2]}', 'autorestart', 'true')
            supervisor_config.set(f'program:{sys.argv[2]}', 'startsecs', '0')
            supervisor_config.set(f'program:{sys.argv[2]}', 'stopwaitsecs', '0')
            supervisor_config.set(f'program:{sys.argv[2]}', 'redirect_stderr', 'true')
        supervisor_config.set(f'program:{sys.argv[2]}', 'directory', f'{getcwd()}')
        supervisor_config.set(f'program:{sys.argv[2]}', 'stdout_logfile',
                   f'{join(join(getcwd(), "logs"), "bomiot_supervisor_access.log")}')
        supervisor_config.set(f'program:{sys.argv[2]}', 'stderr_logfile',
                   f'{join(join(getcwd(), "logs"), "bomiot_supervisor_err.log")}')
        supervisor_config.remove_section('program:bomiot')
        supervisor_config.write(open(join(supervisor_path, str(sys.argv[2]) + '.conf'), "wt"))


        print(f'Deploy project {str(sys.argv[2])} workspace success')



from rest_framework import viewsets
from rest_framework.response import Response
from rest_framework.exceptions import APIException
from functools import reduce

from .serializers import UserSerializer
from .filter import UserFilter
from.page import CorePageNumberPagination
from rest_framework.filters import OrderingFilter
from django_filters.rest_framework import DjangoFilterBackend
from .models import Permission
from .message import permission_message_return

from django.contrib.auth import get_user_model

User = get_user_model()


class UserPage(CorePageNumberPagination):

    def get_return_data(self, data):
        return { "label": permission_message_return(self.request.META.get('HTTP_LANGUAGE'), data.name), "value": data.name }

    def query_data_add(self) -> list:
        permission_list = Permission.objects.all()
        data_list = list(map(lambda data: self.get_return_data(data), permission_list))
        return [
            ("permission", data_list)
        ]


class UserList(viewsets.ModelViewSet):
    """
        list:
            Response a user data list（all）
    """
    pagination_class = UserPage
    filter_backends = [DjangoFilterBackend, OrderingFilter, ]
    ordering_fields = ['id', "created_time", "updated_time", ]
    filter_class = UserFilter

    def get_project(self):
        try:
            data_id = self.kwargs.get('pk')
            return data_id
        except:
            return None

    def get_queryset(self):
        data_id = self.get_project()
        if self.request.user:
            if data_id is None:
                return User.objects.filter(openid=self.request.auth.openid, is_delete=False)
            else:
                return User.objects.filter(openid=self.request.auth.openid, id=data_id, is_delete=False)
        else:
            return User.objects.none()

    def get_serializer_class(self):
        if self.action in ['list']:
            return UserSerializer
        else:
            return self.http_method_not_allowed(request=self.request)


class UserCreate(viewsets.ModelViewSet):
    """
        create:
            create a user
    """
    filter_backends = [DjangoFilterBackend, OrderingFilter, ]
    ordering_fields = ['id', "created_time", "updated_time", ]
    filter_class = UserFilter

    def get_serializer_class(self):
        if self.action in ['create']:
            return UserSerializer
        else:
            return self.http_method_not_allowed(request=self.request)

    def create(self, request, **kwargs):
        data = self.request.data
        print(data)
        return Response(data, status=200)


class UserPermission(viewsets.ModelViewSet):
    """
        create:
            Set permission for user
    """
    filter_backends = [DjangoFilterBackend, OrderingFilter, ]
    ordering_fields = ['id', "created_time", "updated_time", ]
    filter_class = UserFilter

    def get_serializer_class(self):
        if self.action in ['create']:
            return UserSerializer
        else:
            return self.http_method_not_allowed(request=self.request)

    def get_permission_data(self, data):
        permission_data = Permission.objects.filter(name=data).first()
        return {
            permission_data.name: permission_data.api
        }

    def create(self, request, **kwargs):
        data = self.request.data
        message_return('1', '2')
        permission_list = data.get('permission')
        data_list = list(map(lambda data: self.get_permission_data(data), permission_list))
        permission_data = reduce(lambda x, y: {**x , **y}, data_list)
        user_data = User.objects.filter(id=data.get('id')).first()
        user_data.permission = permission_data
        user_data.save()
        return Response({'extra': 'Success Change User Permission'}, status=200)


class UserChangePWD(viewsets.ModelViewSet):
    """
        create:
            Change Password
    """
    filter_backends = [DjangoFilterBackend, OrderingFilter, ]
    ordering_fields = ['id', "created_time", "updated_time", ]
    filter_class = UserFilter

    def get_serializer_class(self):
        if self.action in ['create']:
            return UserSerializer
        else:
            return self.http_method_not_allowed(request=self.request)

    def create(self, request, **kwargs):
        data = self.request.data
        user_data = User.objects.filter(id=data.get('id')).first()
        user_data.set_password(str(data.get('pwd')))
        user_data.save()
        return Response({'extra': 'Success Change Password'}, status=200)


class UserLock(viewsets.ModelViewSet):
    """
        create:
            Delete one User
    """
    filter_backends = [DjangoFilterBackend, OrderingFilter, ]
    ordering_fields = ['id', "created_time", "updated_time", ]
    filter_class = UserFilter

    def get_serializer_class(self):
        if self.action in ['create']:
            return UserSerializer
        else:
            return self.http_method_not_allowed(request=self.request)

    def create(self, request, **kwargs):
        data = self.request.data
        user_data = User.objects.filter(id=data.get('id')).first()
        if user_data.is_superuser:
            raise APIException('Can not lock superuser')
        else:
            if user_data.is_active is True:
                user_data.is_active = False
                user_data.save()
                return Response({'extra': f'Success unlock User {user_data.username}'}, status=200)
            else:
                user_data.is_active = True
                user_data.save()
                return Response({'extra': f'Success lock User {user_data.username}'}, status=200)


class UserDelete(viewsets.ModelViewSet):
    """
        create:
            Delete one User
    """
    filter_backends = [DjangoFilterBackend, OrderingFilter, ]
    ordering_fields = ['id', "created_time", "updated_time", ]
    filter_class = UserFilter

    def get_serializer_class(self):
        if self.action in ['create']:
            return UserSerializer
        else:
            return self.http_method_not_allowed(request=self.request)

    def create(self, request, **kwargs):
        data = self.request.data
        user_data = User.objects.filter(id=data.get('id')).first()
        if user_data.is_superuser:
            raise APIException('Can not delete superuser')
        else:
            user_data.is_delete = True
            user_data.save()
            return Response({'extra': f'Success delete User {user_data.username}'}, status=200)


from django.contrib.auth import get_user_model
from bomiot.server.core.utils import contains_value
import tomllib


User = get_user_model()


class CorePermission:
    def has_permission(self, request, view) -> bool:
        user_permission = request.auth.permission
        if request.user:
            return contains_value(user_permission, request.path)
        else:
            return False
        # return bool(request.user and request.user.is_authenticated)